# encoding: utf-8
import requests
from bs4 import BeautifulSoup
import re
from allabolag.utils import _dl_to_dict, _table_to_dict, _prefix_keys
from allabolag.parsers import PARSERS

class Company():
    """Represents a single company.

    Usage:
        c = Company("559006-6642")
        print(c.data) # get all cleaned data
        print(c.raw_data) # get all uncleaned data
    """
    def __init__(self, company_code):
        self.company_code = company_code
        self.url = "https://www.allabolag.se/{}".format(company_code.replace("-",""))
        self._data = {}
        self._overview_data = {}
        self._activity_data = {}
        self._accounts_data = {}

    @property
    def raw_data(self):
        """Get data from all sections
        """
        data = {}
        data.update(self.overview_data)
        data.update(self.activity_data)
        data.update(self.accounts_data)

        return data

    @property
    def data(self):
        return self._clean_data(self.raw_data)


    @property
    def overview_data(self):
        """Collect data from 'Översikt'
        """
        if self._overview_data == {}:
            s = self._get_soup()
            data = {
                "Namn": s.select_one("h1").text.strip(),

            }

            # Parse "Information" box
            information_elem = s.find(text="Information").parent.parent
            info_list = information_elem.select_one("dl")
            information = _dl_to_dict(info_list)
            data.update(information)

            # Parse "Kontaktuppgifter" box
            elem = s.find(text="Kontaktuppgifter").parent.parent
            dl = elem.select_one("dl")
            contacts = _dl_to_dict(dl)
            data.update(contacts)

            # Parse "Nyckeltal"
            account_fig_summary_soup = s.select_one(".company-account-figures")
            if account_fig_summary_soup is not None:
                data["account_figures_year"] = account_fig_summary_soup.select_one("h2").text.strip()
                account_fig_table =  account_fig_summary_soup.select_one("table")
                keys = [x.text.strip() for x in account_fig_table.select("th")]
                values = [x.text.strip() for x in account_fig_table.select("td")]
                account_figures = dict(zip(keys, values))
                data.update(account_figures)
            data = _prefix_keys(data, u"Översikt")

            self._overview_data = data

        return self._overview_data

    @property
    def activity_data(self):
        """Collect data from "Verksamhet & status"
        """
        if self._activity_data == {}:
            s = self._get_soup("verksamhet")
            data = {}

            # hova in alla dt-dd-taggar
            for dl in s.select("dl"):
                data.update(_dl_to_dict(dl))

            # "Verksamhet & ändamål" följer annan struktur (h3 + initliggande tagg)
            verksamhet_header = s.find(text=re.compile(u"^.*Verksamhet & ändamål.*"))
            if verksamhet_header:
                data[u"Verksamhet & ändamål"] = verksamhet_header.parent.find_next_sibling().text.strip()

            # SNI-koden ligger som kod (dt) + etikett (dd)
            sni_dl = s.select_one(".accordion-body.sni")
            if sni_dl:
                if sni_dl.select_one("dt"):
                    data[u"SNI-kod"] = sni_dl.select_one("dt").text.strip()
                    data[u"SNI-bransch"] = sni_dl.select_one("dd").text.strip()

            data = _prefix_keys(data, "Aktivitet och status")

            self._activity_data = data

        return self._activity_data

    @property
    def accounts_data(self):
        """Collect data from "Bokslut & nyckeltal" """
        if self._accounts_data == {}:
            data = {}
            s = self._get_soup("bokslut")
            for table in s.select("table"):
                #print _table_to_dict(table)
                table_caption = table.select_one("thead th.company-table__pager-button-cell").text.strip()
                if table_caption != u"Nyckeltal":
                    table_data = _table_to_dict(table)
                    table_data = _prefix_keys(table_data, table_caption)
                    data.update(table_data)
                else:
                    # Nyckeltal behöver egen parsing
                    # Hack Nyckeltalstabellen saknar år, därför tar vi dem från föregående år
                    years = [x[0] for x in list(table_data.values())[0]]

                    table_data = {}
                    for tr in table.select("tbody tr"):
                        try:
                            # Celler med
                            key = tr.select_one("span.row-title > span.tooltip > span").text
                        except AttributeError:
                            # Celler utan tooltip
                            key = tr.select_one("th").text.strip()

                        values = [x.text.strip() for x in tr.select("td.data-pager__page")]
                        table_data[key] = list(zip(years, values))

                    table_data = _prefix_keys(table_data, "Nycketal")
                    data.update(table_data)
            self._accounts_data = data
        return self._accounts_data

    def _clean_data(self, dict_):
        for key, value in dict_.items():
            if key in PARSERS:
                parser = PARSERS[key]
                dict_[key] = parser(value)

        return dict_

    def _get_soup(self, endpoint=None):
        url = self.url
        if endpoint:
            url += "/{}".format(endpoint)
        print("/GET {}".format(url))
        r = requests.get(url)
        r.raise_for_status()
        return BeautifulSoup(r.content, "html.parser")



# encoding: utf-8
from datetime import datetime

def value(s):
    """Parse float
    """
    if s is None or s == "-" or s == "":
        return None
    s = s.replace(",",".").replace(" ","").replace(" ","")
    if s.endswith("%"):
        s = s.replace("%", "")
        return float(s) / 100.0
    else:
        return float(s)

    raise Exception(u"Unable to parse value from '{}'".format(s))

def date(s):
    """Parse datetime"""
    return datetime.strptime(s, "%Y-%m-%d")

def date_value(l):
    """Parse [YEAR, VALUE] pair.

    date_value("2018-02", 25%) => [datetime(2018,2,1), 0.25]
    """
    return [
        #datetime.strptime(l[0], "%Y-%m"),
        l[0],
        value(l[1])
    ]

def date_value_list(ll):
    return [date_value(l) for l in ll]

def text(s):
    if s.endswith(u"Läs mer"):
        s = s.replace(u"Läs mer", "").strip()
    return s

# Define parser for specific data fields
PARSERS = {
    u"Aktivitet och status - Anledning till avregistrering": text,
    u"Aktivitet och status - Anmärkning": text,
    u"Aktivitet och status - Bolaget registrerat": text,
    u"Aktivitet och status - Bolagsform": text,
    u"Aktivitet och status - F-Skatt": text,
    u"Aktivitet och status - Kommunsäte": text,
    u"Aktivitet och status - Länsäte": text,
    u"Aktivitet och status - Moms": text,
    u"Aktivitet och status - SNI-bransch": text,
    u"Aktivitet och status - SNI-kod": text,
    u"Aktivitet och status - Slutdatum för F-Skatt": text,
    u"Aktivitet och status - Startdatum för F-Skatt": text,
    u"Aktivitet och status - Startdatum för moms": text,
    u"Aktivitet och status - Status": text,
    u"Aktivitet och status - Verksamhet & ändamål": text,
    u"Aktivitet och status - Ägandeförhållande": text,
    u"Balansräkningar (tkr) - Anläggningstillgångar": date_value_list,
    u"Balansräkningar (tkr) - Avsättningar (tkr)": date_value_list,
    u"Balansräkningar (tkr) - Eget kapital": date_value_list,
    u"Balansräkningar (tkr) - Kortfristiga skulder": date_value_list,
    u"Balansräkningar (tkr) - Långfristiga skulder": date_value_list,
    u"Balansräkningar (tkr) - Obeskattade reserver": date_value_list,
    u"Balansräkningar (tkr) - Omsättningstillgångar": date_value_list,
    u"Balansräkningar (tkr) - Skulder och eget kapital": date_value_list,
    u"Balansräkningar (tkr) - Skulder, eget kapital och avsättningar": date_value_list,
    u"Balansräkningar (tkr) - Tecknat ej inbetalt kapital": date_value_list,
    u"Balansräkningar (tkr) - Tillgångar": date_value_list,
    u"Löner & utdelning (tkr) - Löner till styrelse & VD": date_value_list,
    u"Löner & utdelning (tkr) - Löner till övriga anställda": date_value_list,
    u"Löner & utdelning (tkr) - Omsättning": date_value_list,
    u"Löner & utdelning (tkr) - Sociala kostnader": date_value_list,
    u"Löner & utdelning (tkr) - Utdelning till aktieägare": date_value_list,
    u"Löner & utdelning (tkr) - Varav resultatlön till övriga anställda": date_value_list,
    u"Löner & utdelning (tkr) - Varav tantiem till styrelse & VD": date_value_list,
    u"Nycketal - Antal anställda": date_value_list,
    u"Nycketal - Bruttovinstmarginal": date_value_list,
    u"Nycketal - Du Pont-modellen": date_value_list,
    u"Nycketal - Kassalikviditet": date_value_list,
    u"Nycketal - Nettoomsättning per anställd (tkr)": date_value_list,
    u"Nycketal - Nettoomsättningförändring": date_value_list,
    u"Nycketal - Personalkostnader per anställd (tkr)": date_value_list,
    u"Nycketal - Rörelsekapital/omsättning": date_value_list,
    u"Nycketal - Rörelseresultat, EBITDA": date_value_list,
    u"Nycketal - Soliditet": date_value_list,
    u"Nycketal - Vinstmarginal": date_value_list,
    u"Resultaträkning (tkr) - Nettoomsättning": date_value_list,
    u"Resultaträkning (tkr) - Resultat efter finansnetto": date_value_list,
    u"Resultaträkning (tkr) - Rörelseresultat (EBIT)": date_value_list,
    u"Resultaträkning (tkr) - Årets resultat": date_value_list,
    u"Resultaträkning (tkr) - Övrig omsättning": date_value_list,
    u"Översikt - Anmärkning": text,
    u"Översikt - Besöksadress": text,
    u"Översikt - Bolagsform": text,
    u"Översikt - F-Skatt": text,
    u"Översikt - Län": text,
    u"Översikt - Moms": text,
    u"Översikt - Omsättning": value,
    u"Översikt - Ort": text,
    u"Översikt - Registreringsår": text,
    u"Översikt - Res. e. fin": text,
    u"Översikt - Summa tillgångar": value,
    u"Översikt - Telefon": text,
    u"Översikt - Name": text,
    u"Översikt - Årets resultat": value,
}



import requests
from bs4 import BeautifulSoup
from datetime import datetime
from copy import deepcopy
import re
import json
from allabolag.utils import _dl_to_dict


def iter_liquidated_companies(until):
    """Iterate the paginated list of liquidated companies ("konkurs inledd").
    The website does not allow us to fetch a given date (range). We may only
    collect the latest records.

    :param until: date of oldest liquidation.
    """
    if not isinstance(until, datetime):
        until = datetime.strptime(until, "%Y-%m-%d")
    page = 1
    has_more_results = True

    while has_more_results:
        url = "https://www.allabolag.se/lista/konkurs-inledd/6/?page={}".format(page)
        print("/GET {}".format(url))
        r = requests.get(url)
        r.raise_for_status()
        soup = BeautifulSoup(r.content, "html.parser")
        data = json.loads(soup.select_one(".page.search-results search")\
                              .attrs[":search-result-default"])

        if len(data) == 0:
            raise Exception(u"No results on {}".format(url))

        for item in data:
            item_data = _parse_liquidated_company_item(item)
            if item_data["Konkurs inledd"] < until:
                has_more_results = False
            else:
                yield item_data

        page += 1

def _parse_liquidated_company_item(item_dict):
    item = deepcopy(item_dict)

    # store for backward compability
    item["link"] = item_dict["linkTo"]
    item["Org.nummer"] = item_dict["orgnr"]

    for remark in item_dict["remarks"]:
        key = remark["remarkDescription"] # ie. Konkurs inledd
        item[key] = datetime.strptime(remark["remarkDate"], "%Y-%m-%d")

    return item




def _dl_to_dict(dl):
    """
    """
    d = {}
    for dt in dl.select("dt"):
        key = dt.text.strip()
        # Assumption that the value is the next sibling
        # this is not always a dd tag (sometimes li tag for example)
        value = dt.find_next_sibling().text.strip()
        d[key] = value
    return d

def _prefix_keys(old_dict, prefix):
    """Adds a prefix to all keys in a dict"""
    new_dict = {}
    for key, value in old_dict.items():
        new_key = u"{} - {}".format(prefix, key)
        new_dict[new_key] = old_dict[key]
    return new_dict

def _table_to_dict(table):
    data = {}
    years = [x.text.strip() for x in table.select("thead th.data-pager__page")]
    for tr in table.select("tbody tr"):
        try:
            key = tr.select_one("th").text.strip()
        except:
            continue
        values = [x.text.strip() for x in tr.select("td.data-pager__page")]
        data[key] = list(zip(years, values))
    return data



# encoding:utf-8
from setuptools import setup

###
# Versioning
###
from datetime import date

name = "allabolag"

short_version = "0.1.5"
long_version = short_version

short_desc = """\
Scrape data from allabolag.se.\
"""
authors = u"Jens Finnäs, Newsworthy"
year = date.today().year
copyright = "%s, %s" % (year, authors)
email = "jens.finnas@gmail.com"

version = long_version

###
# Setup
###

def readme():
    """Import README for use as long_description."""
    with open("README.rst") as f:
        return f.read()


repo = "https://github.com/marple-newsrobot/allabolag"

setup(
    name=name,
    version=version,
    description=short_desc,
    long_description=readme(),
    long_description_content_type='text/x-rst',
    url=repo,
    author=authors,
    author_email=email,
    license="MIT",
    packages=[name],
    zip_safe=False,
    python_requires='>=2.7',
    install_requires=[
        "requests>=2.22.0",
        "beautifulsoup4>=4.7.1",
    ],
    include_package_data=True,
    download_url="{}/archive/{}.tar.gz".format(repo, version),
)
